/* 
 *   FILE: wnck.c --
 * AUTHOR: Martin Man <mman@martinman.net>
 *   DATE: 28 December 2005
 *
 * Copyright (c) 2005 Martin Man.
 *
 * This application is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This application is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this application; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "src/wnck.h"

/**
 * Call given function for each and every element in the given list and
 * return new list containing elements that passed the check.
 *
 * @param list a GList
 * @param b data that will be passed to func as argument 'b'
 * @param data data that will be passed to func as argument 'b'
 * @param func method to be called for each element of list, every element of
 * the list is compared with given data.
 * @return new list of elements for which calling a func
 * returned zero.
 */
GList *
g_list_map( GList * list,
        gpointer b, gpointer data, GCompareDataFunc func )
{
    GList * i = NULL;
    GList * ret = NULL;

    /* list, b, and data can be null */
    /* g_return_val_if_fail( b != NULL ); */
    /* g_return_val_if_fail( data != NULL ); */
    g_return_val_if_fail( func != NULL, NULL );

    for( i = list; i != NULL; i = g_list_next( i ) ) {
        /* printf( "g_list_map: i->data is 0x%x\n", (unsigned)i->data ); */
        if( func( i->data, b, data ) == 0 ) {
            /* printf( "g_list_map: map func succeeded\n" ); */
            ret = g_list_append( ret, i->data );
        }
    }

    return ret;
}

/**
 * Call given function for each and every element of given lists and return
 * 0 when both lists are the same (the same data in the same order).
 *
 * @param a a GList
 * @param b a GList
 * @param func method to be called for each element of both lists.
 * @return 0 when both lists contain the same elements in the same order.
 */
gint
g_list_compare( GList * a, GList * b,
        GCompareFunc func )
{
    GList * i = NULL;
    GList * j = NULL;

    i = a;
    j = b;

    while( i != NULL && j != NULL ) {
        if( func( i->data, j->data ) != 0 ) {
            return -1;
        }
        i = g_list_next( i );
        j = g_list_next( j );
    }

    if( i != NULL || j != NULL ) {
        return -1;
    }

    return 0;
}

/**
 * Print given list of strings.
 */
void
window_list_print( GList * list, const gchar * caption )
{
    GList * i = NULL;

    printf( "%s", caption );

    for( i = list; i != NULL; i = g_list_next( i ) ) {
        printf( "%s\n",
               wnck_window_get_name( WNCK_WINDOW( i->data ) ) );
    }
}

/**
 * Check window state.
 *
 * @param a a WnckWindow whose state should be checked
 * @param b ignored
 * @param data WnckWindowState flagset that should be excluded
 * @return 0 if the window does not have given flags set
 */
static gint
_check_wstate( gconstpointer a, gconstpointer b, gpointer data )
{
    g_assert( WNCK_IS_WINDOW( a ) );

    /* printf( "_check_wstate: a is %s\n",
            wnck_window_get_name( WNCK_WINDOW( a ) ) ); */

    WnckWindowState ws = wnck_window_get_state( WNCK_WINDOW( a ) );

    if( ( ws & GPOINTER_TO_INT( data ) ) != 0 ) {
        return 1;
    }

    return 0;
}

/**
 * Return the list of windows on the given screen, excluding windows that
 * have the given window state flags set.
 *
 * @param list a GList of window titles (const gchar*)
 * @param ws a set of flags that should be excluded
 * @return new list of windows
 */
GList *
window_list_get( WnckScreen * screen, WnckWindowState ws )
{
    GList * all = NULL;

    g_return_val_if_fail( screen != NULL, NULL );

    all = wnck_screen_get_windows( screen );
    return g_list_map( all,
            NULL, GINT_TO_POINTER( ws ), _check_wstate );
}

/**
 * Check window title.
 *
 * @param a a title to be checked (const gchar*)
 * @param b a pattern (const gchar*)
 * @param data unused
 * @return 0 when a matches b (uses g_strstr ATM).
 */
static gint
_check_title( gconstpointer a, gconstpointer b, gpointer data )
{
    gchar * title = NULL;

    g_assert( WNCK_IS_WINDOW( a ) );
    g_assert( b != NULL );

    title = g_utf8_strdown(
            wnck_window_get_name( WNCK_WINDOW( a ) ), -1 );

    if ( g_strrstr( title, (const gchar*) b ) != NULL ) {
        g_free( title );
        return 0;
    }

    g_free( title );
    return 1;
}

/**
 * Find windows with given pattern matching titles.
 *
 * @param list a GList of window titles (const gchar*)
 * @param pattern a pattern to be applied to the window titles (const
 * gchar*)
 * @return new list of windows
 */
GList *
window_list_filter( GList * list, const gchar * pattern )
{
    GList * ret = NULL;
    gchar * pt = g_utf8_strdown( pattern, -1 );

    ret = g_list_map( list, (gpointer) pt, NULL, _check_title );

    g_free( pt );
    return ret;
}

/* vim: set et ts=4 sw=4 tw=76:
 */
